# GocustomUrls

This is a homespun implementation of the practice of [using custom import paths for go modules](https://pkg.go.dev/cmd/go#hdr-Remote_import_paths).

This package basically returns a html document with a prefilled meta tag

```html
<meta name="go-import" content="{package} git {url location of the package}">
```

There are ways to do this with [nginx](https://www.nirenjan.com/2019/golang-vanity-urls-in-nginx/) or [hugo](https://blog.jbowen.dev/2020/07/using-go-vanity-urls-with-hugo/) but I wanted to:

* host on a subdomain
* not muck up my ngnix config

So I used golang for this project.

## Running

```sh
$ ./artifacts/gocustomurls --conf <PATH_TO_CONF_FILE>
```
A sample config file is located [here](./sample/config.json) and a sample rules file is located [here](./sample/rules.json)

## SystemD

A sample systemd file is located [here](./sample/plain/gocustomurls.service). You should run this set of commands from [here](./sample/plain/commands.md)


## Docker

A sample Dockefile for use with systemd is [here](./sample/withDocker/Dockerfile.1). You should run this set of commands from [here](./sample/withDocker/commands.md).

If you do not want to use systemd, you can run this instead:

```sh
$ HOME=${PWD} docker build -t appimage/gocustomurls --build-arg UID=$(id -u) --build-arg GID=$(id -g) --build-arg USERNAME=$(whoami) --build-arg PORT=7070 --no-cache --progress=plain .
$ 
```

## Testing

You can test with 

(a) [httpie](https://httpie.io/)

```sh
$ http --body "https://{domain.name}/{package}?go-get=1"
...Truncated output
```

(b) With the go-get command

```sh
$ go get -v -u jbowen.dev/cereal
get "jbowen.dev/cereal": found meta tag get.metaImport{Prefix:"jbowen.dev/cereal", VCS:"git", RepoRoot:"https://github.com/jamesbo13/cereal"} at //jbowen.dev/cereal?go-get=1
jbowen.dev/cereal (download)
```

## TODOs

* [x] Fix permission errors around opening the app.log and rules.json.
* [x] Make the flags (config, rules) required instead of optional.
* [x] ~~Figure how to use logrotate (a linux utility)~~
* [x] Figure how to do log rotation as part of this app's function
* [x] Add tests
* [x] Add systemd.service and explanation
* [x] Add Dockerfile and explanation
* [x] Add mirror to Github
* [ ] Add Github action
* [ ] Figure how to use `goreleaser` [here](https://nfpm.goreleaser.com/) to release deb and rpm packages (so basically split it, goreleaser for github and woodpecker for Woodpecker)
* [ ] Update README.md
